package org.prlab.idic.privacypolicy;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.widget.RecyclerView;
import android.view.View;
import android.widget.TextView;
import org.prlab.idic.privacypolicy.adapter.InfoItemRecyclerViewAdapter;
import org.prlab.idic.privacypolicy.adapter.StatementRecyclerViewAdapter;
import org.prlab.idic.privacypolicy.pojo.report.InfoItem;
import org.prlab.idic.privacypolicy.pojo.report.PrivacyPolicy;
import org.prlab.idic.privacypolicy.tool.BaseUtils;

import java.util.LinkedList;
import java.util.List;

/**
 * Created by IDIC on 2017/2/17.
 */
public class DialogPolicyDetailActivity extends Activity {

    private PrivacyPolicy mPrivacyPolicy;
    private RecyclerView mCollectorRecyclerView;
    private TextView mAccessTextView;
    private RecyclerView mDisputeRecyclerView;
    private RecyclerView mStatementsRecyclerView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_dialog_policy);
        mPrivacyPolicy = getIntent().getParcelableExtra(getString(R.string.intent_policy_info_key));
        setTitle(getString(R.string.message_policy_title) + mPrivacyPolicy.getId());
        initView();
    }

    private void initView() {
        initCollector();
        initAccess();
        initDispute();
        initStatements();
    }

    private void initCollector() {
        mCollectorRecyclerView = (RecyclerView) findViewById(R.id.recycler_view_statement_collector);
        List<InfoItem> collector = new LinkedList<>();
        collector.add(new InfoItem()
                .setKey(getString(R.string.policy_statement_collector_name))
                .setValue(mPrivacyPolicy.getCollector().getName()));
        collector.add(new InfoItem()
                .setKey(getString(R.string.policy_statement_collector_email))
                .setValue(mPrivacyPolicy.getCollector().getEmail()));
        collector.add(new InfoItem()
                .setKey(getString(R.string.policy_statement_collector_phone))
                .setValue(mPrivacyPolicy.getCollector().getPhone()));
        mCollectorRecyclerView.setAdapter(new InfoItemRecyclerViewAdapter(collector));
    }

    private void initAccess() {
        mAccessTextView = (TextView) findViewById(R.id.recycler_view_statement_access);
        mAccessTextView.setText(BaseUtils.fixTypeName(mPrivacyPolicy.getAccess().name()));
    }

    private void initDispute() {
        mDisputeRecyclerView = (RecyclerView) findViewById(R.id.recycler_view_statement_dispute);
        List<InfoItem> dispute = new LinkedList<>();
        dispute.add(new InfoItem()
                .setKey(getString(R.string.policy_statement_dispute_resolution_type))
                .setValue(getString(R.string.policy_statement_dispute_related_organization)));
        dispute.add(new InfoItem()
                .setKey(BaseUtils.fixTypeName(mPrivacyPolicy.getDispute().getType().name()))
                .setValue(mPrivacyPolicy.getDispute().getRelatedOrganization()));
        mDisputeRecyclerView.setAdapter(new InfoItemRecyclerViewAdapter(dispute));
    }

    private void initStatements() {
        mStatementsRecyclerView = (RecyclerView) findViewById(R.id.recycler_view_statement_content);
        mStatementsRecyclerView.setAdapter(new StatementRecyclerViewAdapter(getResources(), mPrivacyPolicy.getStatements()));
    }

    public void onAgreeButtonClick(View view) {
        Intent intent = new Intent();
        intent.putExtra(getString(R.string.intent_policy_response_selection_key), mPrivacyPolicy.getId());
        setResult(RESULT_OK, intent);
        finish();
    }

    public void onDenyButtonClick(View view) {
        finish();
    }

}
