package org.prlab.idic.privacypolicy;

import android.app.Activity;
import android.app.LoaderManager;
import android.content.AsyncTaskLoader;
import android.content.Intent;
import android.content.Loader;
import android.os.Bundle;
import android.support.v7.widget.RecyclerView;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;
import com.google.gson.Gson;
import okhttp3.FormBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import org.prlab.idic.privacypolicy.adapter.IntegerButtonRecyclerViewAdapter;
import org.prlab.idic.privacypolicy.adapter.StringButtonRecyclerViewAdapter;
import org.prlab.idic.privacypolicy.pojo.report.Layer;
import org.prlab.idic.privacypolicy.pojo.report.PrivacyPolicyReport;
import org.prlab.idic.privacypolicy.storage.PrivacyPolicyStorageImpl;
import org.prlab.idic.privacypolicy.tool.BaseUtils;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import static org.prlab.idic.privacypolicy.tool.NegotiatorConstants.ACTIVITY_SELECTION_RESULT_ID;

/**
 * Created by IDIC on 2017/2/16.
 */
public class ReportActivity extends Activity implements LoaderManager.LoaderCallbacks<PrivacyPolicyReport> {

    private LinearLayout mReportLayout;
    private TextView mReportTitleTextView;
    private TextView mVersionTextView;
    private TextView mDescriptionTextView;
    private Button mDeviceInfoButton;

    private Button mResetButton;
    private TextView mSelectionTitleTextView;
    private RecyclerView mSelectionRecyclerView;
    private IntegerButtonRecyclerViewAdapter mSelectionRecyclerViewAdapter;

    private TextView mLayerTitleTextView;
    private RecyclerView mLayerRecyclerView;
    private StringButtonRecyclerViewAdapter mLayerRecyclerViewAdapter;

    private Button mBackButton;

    private Gson gson = new Gson();

    private PrivacyPolicyStorageImpl mPolicyStorage;
    private List<String> mLayerIDs;
    private PrivacyPolicyReport mPrivacyPolicyReport;
    private String mReportId;

    private List<Integer> mPolicyIDs;
    private List<String> mChildLayerIDs;

    private static final int PRIVACY_POLICY_LOADER_ID = 87;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_report);
        resolveData();
        initView();
    }

    private void resolveData() {
        mLayerIDs = new LinkedList<>();

        Intent intent = getIntent();
        mReportId = intent.getStringExtra(getString(R.string.intent_report_id_key));
        mPolicyStorage = new PrivacyPolicyStorageImpl(getContentResolver(), intent.getStringExtra(getString(R.string.intent_report_mac_key)));
        getLoaderManager().initLoader(PRIVACY_POLICY_LOADER_ID, null, this);
    }

    private void initView() {
        mReportLayout = (LinearLayout) findViewById(R.id.layout_report_info);
        mReportTitleTextView = (TextView) findViewById(R.id.text_report_title);

        View mVersionView = findViewById(R.id.item_report_version);
        ((TextView) mVersionView.findViewById(R.id.text_key)).setText(getString(R.string.policy_report_version_title));
        mVersionTextView = (TextView) mVersionView.findViewById(R.id.text_value);

        View mDescriptionView = findViewById(R.id.item_report_description);
        ((TextView) mDescriptionView.findViewById(R.id.text_key)).setText(getString(R.string.policy_report_description_title));
        mDescriptionTextView = (TextView) mDescriptionView.findViewById(R.id.text_value);

        mDeviceInfoButton = (Button) findViewById(R.id.button_report_device);
        mBackButton = (Button) findViewById(R.id.button_report_back);

        mResetButton = (Button) findViewById(R.id.button_report_reset);
        mResetButton.setOnClickListener(mResetButtonClickListener);

        mSelectionTitleTextView = (TextView) findViewById(R.id.text_report_policies_title);
        mSelectionRecyclerViewAdapter = new IntegerButtonRecyclerViewAdapter(getResources(), mSelectionClickListener);
        mSelectionRecyclerView = (RecyclerView) findViewById(R.id.recycler_view_report_policies);
        mSelectionRecyclerView.setAdapter(mSelectionRecyclerViewAdapter);

        mLayerTitleTextView = (TextView) findViewById(R.id.text_report_child_layers_title);
        mLayerRecyclerViewAdapter = new StringButtonRecyclerViewAdapter(getResources(), mChildLayersClickListener);
        mLayerRecyclerView = (RecyclerView) findViewById(R.id.recycler_view_report_child_layers);
        mLayerRecyclerView.setAdapter(mLayerRecyclerViewAdapter);
    }

    private IntegerButtonRecyclerViewAdapter.ListItemClickListener mSelectionClickListener = new IntegerButtonRecyclerViewAdapter.ListItemClickListener() {
        @Override
        public void onListItemClick(int clickedItemIndex) {
            Intent intent = new Intent();
            intent.setClass(ReportActivity.this, DialogPolicyDetailActivity.class);
            int policyId = mPolicyIDs.get(clickedItemIndex);
            intent.putExtra(getString(R.string.intent_policy_info_key), mPrivacyPolicyReport.getPolicies().get(policyId));
            startActivityForResult(intent, ACTIVITY_SELECTION_RESULT_ID);
        }
    };

    private StringButtonRecyclerViewAdapter.ListItemClickListener mChildLayersClickListener = new StringButtonRecyclerViewAdapter.ListItemClickListener() {
        @Override
        public void onListItemClick(int clickedItemIndex) {
            String layerID = mChildLayerIDs.get(clickedItemIndex);
            mLayerIDs.add(layerID);
            setView();
        }
    };

    private View.OnClickListener mResetButtonClickListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {
            mPolicyStorage.remove(mLayerIDs);
            mResetButton.setEnabled(false);
            mSelectionRecyclerViewAdapter.setTarget(-1);
        }
    };

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (ACTIVITY_SELECTION_RESULT_ID == requestCode) {
            if (RESULT_OK == resultCode) {
                int selection = data.getIntExtra(getString(R.string.intent_policy_response_selection_key), -1);
                mPolicyStorage.put(selection, mLayerIDs);
                mResetButton.setEnabled(true);
                mSelectionRecyclerViewAdapter.setTarget(selection);
            }
        }
    }

    @Override
    public Loader<PrivacyPolicyReport> onCreateLoader(int id, Bundle args) {
        return new AsyncTaskLoader<PrivacyPolicyReport>(this) {

            @Override
            protected void onStartLoading() {
                if (null == mPrivacyPolicyReport) {
                    forceLoad();
                }
            }

            @Override
            public PrivacyPolicyReport loadInBackground() {
                OkHttpClient client = new OkHttpClient();
                Request request = new Request.Builder()
                        .url(getString(R.string.get_policy_url))
                        .post(new FormBody.Builder()
                                .add(getString(R.string.get_policy_id_parameter_key), mReportId)
                                .build())
                        .build();
                try {
                    return gson.fromJson(client.newCall(request).execute().body().string(), PrivacyPolicyReport.class);
                } catch (IOException e) {
                    e.printStackTrace();
                    return null;
                }
            }

        };
    }

    @Override
    public void onLoadFinished(Loader<PrivacyPolicyReport> loader, PrivacyPolicyReport report) {
        if (null != report) {
            mPrivacyPolicyReport = report;
            mLayerIDs.add(report.getRoot().getUUID());
            setView();
        } else {
            Toast.makeText(this, R.string.message_report_not_found, Toast.LENGTH_LONG).show();
            finish();
        }
    }

    @Override
    public void onLoaderReset(Loader<PrivacyPolicyReport> loader) {

    }

    private void setView() {
        Layer currentLayer = currentLayer();
        String title;
        if (mPrivacyPolicyReport.getRoot().equals(currentLayer)) {
            mVersionTextView.setText(mPrivacyPolicyReport.getVersion());
            mDescriptionTextView.setText(mPrivacyPolicyReport.getDescription());
//            mReportLayout.setVisibility(View.VISIBLE);
//            mDeviceInfoButton.setVisibility(View.VISIBLE);
            mBackButton.setVisibility(View.GONE);
            title = getString(R.string.policy_information_title);
        } else {
            title = currentLayer.getUUID();
//            mReportLayout.setVisibility(View.GONE);
//            mDeviceInfoButton.setVisibility(View.GONE);
            mBackButton.setVisibility(View.VISIBLE);
        }
        mReportTitleTextView.setText(title);
        mPolicyIDs = currentLayer.getPolicies();
        mSelectionRecyclerViewAdapter.setItems(mPolicyIDs);
        int target = mPolicyStorage.get(mLayerIDs);
        mResetButton.setEnabled(target != -1);
        mSelectionRecyclerViewAdapter.setTarget(target);
        checkEmpty(mPolicyIDs, mSelectionTitleTextView, mSelectionRecyclerView, mResetButton);

        mChildLayerIDs = BaseUtils.layer2UUID(currentLayer.getSubLayers());
        mLayerRecyclerViewAdapter.setItems(mChildLayerIDs);
        checkEmpty(mChildLayerIDs, mLayerTitleTextView, mLayerRecyclerView);
    }

    private void checkEmpty(List list, View title, View... views) {
        if (list.isEmpty()) {
            title.setVisibility(View.GONE);
            for (View view : views)
                view.setVisibility(View.GONE);
        } else {
            title.setVisibility(View.VISIBLE);
            for (View view : views)
                view.setVisibility(View.VISIBLE);
        }
    }

    private Layer currentLayer() {
        return currentLayer(mPrivacyPolicyReport.getRoot(), 0);
    }

    private Layer currentLayer(Layer root, int counter) {
        if (mLayerIDs.size() - 1 == counter) return root;
        for (Layer subLayer : root.getSubLayers()) {
            if (subLayer.getUUID().equals(mLayerIDs.get(counter + 1))) return currentLayer(subLayer, counter + 1);
        }
        return root;
    }

    public void onDeviceInfoButtonClicked(View view) {
        Intent intent = new Intent();
        intent.setClass(ReportActivity.this, DialogDeviceInfoActivity.class);
        intent.putExtra(getString(R.string.intent_device_info_key), mPrivacyPolicyReport.getDevice());
        startActivity(intent);
    }

    public void onBackButtonClicked(View view) {
        mLayerIDs.remove(mLayerIDs.size() - 1);
        setView();
    }

}
