package org.prlab.idic.privacypolicy.adapter;

import android.content.res.Resources;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import org.prlab.idic.privacypolicy.R;

import java.util.List;

/**
 * Created by IDIC on 2017/2/16.
 */
public class IntegerButtonRecyclerViewAdapter extends RecyclerView.Adapter<IntegerButtonRecyclerViewAdapter.ViewHolder> {

    private final String PRIVACY_POLICY_TITLE;
    private ListItemClickListener listItemClickListener;
    private List<Integer> items;
    private Integer target = -1;

    public IntegerButtonRecyclerViewAdapter(Resources resources, ListItemClickListener listItemClickListener) {
        this.PRIVACY_POLICY_TITLE = resources.getString(R.string.message_policy_title);
        this.listItemClickListener = listItemClickListener;
    }

    public void setItems(List<Integer> items) {
        this.items = items;
        notifyDataSetChanged();
    }

    public void setTarget(Integer target) {
        int index = items.indexOf(this.target);
        if (index != -1)
            notifyItemChanged(index);

        this.target = target;

        index = items.indexOf(this.target);
        if (index != -1)
            notifyItemChanged(index);
    }

    public interface ListItemClickListener {
        void onListItemClick(int clickedItemIndex);
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.recycler_view_item, parent, false);
        return new IntegerButtonRecyclerViewAdapter.ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        holder.bind(position);
    }

    @Override
    public int getItemCount() {
        return null != items ? items.size() : 0;
    }

    class ViewHolder extends RecyclerView.ViewHolder {

        private Button button;

        ViewHolder(View itemView) {
            super(itemView);
            button = (Button) itemView.findViewById(R.id.button_item);
            button.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    IntegerButtonRecyclerViewAdapter.this.listItemClickListener.onListItemClick(getAdapterPosition());
                }
            });
        }

        void bind(int listIndex) {
            int item = items.get(listIndex);
            String message = IntegerButtonRecyclerViewAdapter.this.PRIVACY_POLICY_TITLE + String.valueOf(item);
            button.setText(message);
            button.setEnabled(item != target);
        }

    }
}
