package org.prlab.idic.privacypolicy.adapter;

import android.content.res.Resources;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import org.prlab.idic.privacypolicy.R;
import org.prlab.idic.privacypolicy.ble.BluetoothDeviceInformation;

import java.util.List;

/**
 * Created by IDIC on 2017/1/20.
 */
public class ScanResultRecyclerViewAdapter extends RecyclerView.Adapter<ScanResultRecyclerViewAdapter.ViewHolder> {

    private static final String TAG = ScanResultRecyclerViewAdapter.class.getSimpleName();

    private final String BONDED_STRING;
    private final String NOT_BONDED_STRING;
    final private ListItemClickListener mOnPolicyImageClickListener;
    final private ListItemClickListener mOnConnectButtonClickListener;
    final private String RSSI_UNIT;
    private List<BluetoothDeviceInformation> devices;

    public interface ListItemClickListener {
        void onListItemClick(int clickedItemIndex);
    }

    public ScanResultRecyclerViewAdapter(Resources resources,
                                         ListItemClickListener mOnPolicyImageClickListener,
                                         ListItemClickListener mOnConnectButtonClickListener,
                                         List<BluetoothDeviceInformation> devices) {
        this.BONDED_STRING = resources.getString(R.string.bluetooth_bonded);
        this.NOT_BONDED_STRING = resources.getString(R.string.bluetooth_not_bonded);
        this.RSSI_UNIT = " " + resources.getString(R.string.unit_rssi);
        this.mOnPolicyImageClickListener = mOnPolicyImageClickListener;
        this.mOnConnectButtonClickListener = mOnConnectButtonClickListener;
        this.devices = devices;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.recycler_view_scan_item, parent, false);
        return new ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        holder.bind(position);
    }

    @Override
    public int getItemCount() {
        return devices.size();
    }

    class ViewHolder extends RecyclerView.ViewHolder {

        private ImageView mPolicyButton;
        private TextView mNameTextView;
        private TextView mMACTextView;
        private TextView mBondedTextView;
        private TextView mRssiTextView;
        private Button mConnectButton;

        ViewHolder(View itemView) {
            super(itemView);
            mPolicyButton = (ImageView) itemView.findViewById(R.id.image_scan_allowed);
            mNameTextView = (TextView) itemView.findViewById(R.id.text_scan_device_name);
            mMACTextView = (TextView) itemView.findViewById(R.id.text_scan_device_mac);
            mBondedTextView = (TextView) itemView.findViewById(R.id.text_scan_is_bonded);
            mRssiTextView = (TextView) itemView.findViewById(R.id.text_scan_rssi);
            mConnectButton = (Button) itemView.findViewById(R.id.button_scan_connect);
        }

        private View.OnClickListener mOnPolicyClickAdapter = new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mOnPolicyImageClickListener.onListItemClick(getAdapterPosition());
            }
        };

        private View.OnClickListener mOnConnectClickAdapter = new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                mOnConnectButtonClickListener.onListItemClick(getAdapterPosition());
            }
        };

        void bind(int listIndex) {
            BluetoothDeviceInformation device = devices.get(listIndex);
            mPolicyButton.setImageResource(null != device.getReportUUID() ? R.drawable.rounded_yes : R.drawable.rounded_no);
            mPolicyButton.setOnClickListener(mOnPolicyClickAdapter);
            mNameTextView.setText(String.valueOf(device.getName()));
            mMACTextView.setText(String.valueOf(device.getAddress()));
            mBondedTextView.setText(device.isBonded() ? BONDED_STRING : NOT_BONDED_STRING);
            mRssiTextView.setText(String.valueOf(device.getRssi() + RSSI_UNIT));
            mConnectButton.setOnClickListener(mOnConnectClickAdapter);
        }
    }
}
