package org.prlab.idic.privacypolicy.adapter;

import android.content.res.Resources;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import org.prlab.idic.privacypolicy.R;
import org.prlab.idic.privacypolicy.pojo.report.InfoItem;
import org.prlab.idic.privacypolicy.pojo.report.p3p.*;
import org.prlab.idic.privacypolicy.tool.BaseUtils;

import java.util.LinkedList;
import java.util.List;

/**
 * Created by IDIC on 2017/2/23.
 */
public class StatementRecyclerViewAdapter extends RecyclerView.Adapter<StatementRecyclerViewAdapter.ViewHolder> {

    private final String CONSEQUENCE_TITLE;
    private final String PURPOSE_TYPE_TITLE;
    private final String PURPOSE_DESCRIPTION_TITLE;
    private final String DATUM_TYPE_TITLE;
    private final String DATUM_DESCRIPTION_TITLE;
    private final String REMEDIES_TITLE;
    private final String RECIPIENT_TYPE_TITLE;
    private final String RECIPIENT_DESCRIPTION_TITLE;
    private final String RETENTION_TITLE;

    private List<Statement> statements;

    public StatementRecyclerViewAdapter(Resources resources, List<Statement> statements) {
        CONSEQUENCE_TITLE = resources.getString(R.string.policy_statement_consequence_title);

        PURPOSE_TYPE_TITLE = resources.getString(R.string.policy_statement_purpose_type);
        PURPOSE_DESCRIPTION_TITLE = resources.getString(R.string.policy_statement_purpose_description);

        DATUM_TYPE_TITLE = resources.getString(R.string.policy_statement_data_type);
        DATUM_DESCRIPTION_TITLE = resources.getString(R.string.policy_statement_data_description);

        REMEDIES_TITLE = resources.getString(R.string.policy_statement_remedies);

        RECIPIENT_TYPE_TITLE = resources.getString(R.string.policy_statement_recipient_type);
        RECIPIENT_DESCRIPTION_TITLE = resources.getString(R.string.policy_statement_recipient_description);

        RETENTION_TITLE = resources.getString(R.string.policy_statement_retention);
        this.statements = statements;
    }

    @Override
    public StatementRecyclerViewAdapter.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.recycler_view_statement, parent, false);
        return new StatementRecyclerViewAdapter.ViewHolder(view);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        holder.bind(position);
    }

    @Override
    public int getItemCount() {
        return null == statements ? 0 : statements.size();
    }

    class ViewHolder extends RecyclerView.ViewHolder {

        private TextView mConsequenceTextView;
        private RecyclerView mPurposesRecyclerView;
        private RecyclerView mDataRecyclerView;
        private TextView mRemediesTextView;
        private RecyclerView mRecipientsRecyclerView;
        private TextView mRetentionTextView;

        ViewHolder(View itemView) {
            super(itemView);
            mConsequenceTextView = (TextView) itemView.findViewById(R.id.recycler_view_statement_consequence);
            mPurposesRecyclerView = (RecyclerView) itemView.findViewById(R.id.recycler_view_statement_purposes);
            mDataRecyclerView = (RecyclerView) itemView.findViewById(R.id.recycler_view_statement_data);
            mRemediesTextView = (TextView) itemView.findViewById(R.id.text_statement_remedies);
            mRecipientsRecyclerView = (RecyclerView) itemView.findViewById(R.id.recycler_view_statement_recipients);
            mRetentionTextView = (TextView) itemView.findViewById(R.id.text_statement_retention);
        }

        private void bindConsequence(String consequence) {
            mConsequenceTextView.setText(consequence);
        }

        private void bindPurposes(List<Purpose> purposes) {
            List<InfoItem> infoItems = new LinkedList<>();
            infoItems.add(new InfoItem()
                    .setKey(PURPOSE_TYPE_TITLE)
                    .setValue(PURPOSE_DESCRIPTION_TITLE));
            for (Purpose purpose : purposes) {
                infoItems.add(new InfoItem()
                        .setKey(BaseUtils.fixTypeName(purpose.getType().name()))
                        .setValue(purpose.getDescription()));
            }
            mPurposesRecyclerView.setAdapter(new InfoItemRecyclerViewAdapter(infoItems));
        }

        private void bindData(List<Datum> data) {
            List<InfoItem> infoItems = new LinkedList<>();
            infoItems.add(new InfoItem()
                    .setKey(DATUM_TYPE_TITLE)
                    .setValue(DATUM_DESCRIPTION_TITLE));
            for (Datum datum : data) {
                infoItems.add(new InfoItem()
                        .setKey(BaseUtils.fixTypeName(datum.getType().name()))
                        .setValue(datum.getDescription()));
            }
            mDataRecyclerView.setAdapter(new InfoItemRecyclerViewAdapter(infoItems));
        }

        private void bindRemedies(List<Remedy> remedies) {
            StringBuilder builder = new StringBuilder();
            for (Remedy remedy : remedies) {
                builder.append(BaseUtils.fixTypeName(remedy.name())).append(", ");
            }
            builder.delete(builder.length() - 2, builder.length());
            mRemediesTextView.setText(builder.toString());
        }

        private void bindRecipients(List<Recipient> recipients) {
            List<InfoItem> infoItems = new LinkedList<>();
            infoItems.add(new InfoItem()
                    .setKey(RECIPIENT_TYPE_TITLE)
                    .setValue(RECIPIENT_DESCRIPTION_TITLE));
            for (Recipient recipient : recipients) {
                infoItems.add(new InfoItem()
                        .setKey(BaseUtils.fixTypeName(recipient.getType().name()))
                        .setValue(recipient.getDescription()));
            }
            mRecipientsRecyclerView.setAdapter(new InfoItemRecyclerViewAdapter(infoItems));
        }

        private void bindRetention(Retention retention) {
            mRetentionTextView.setText(BaseUtils.fixTypeName(retention.name()));
        }

        void bind(int listIndex) {
            Statement statement = statements.get(listIndex);
            bindConsequence(statement.getConsequence());
            bindPurposes(statement.getPurposes());
            bindData(statement.getData());
            bindRemedies(statement.getRemedies());
            bindRecipients(statement.getRecipients());
            bindRetention(statement.getRetention());
        }

    }
}
