package org.prlab.idic.privacypolicy.pojo.report.device;

import android.os.Parcel;
import android.os.Parcelable;

import java.util.LinkedList;
import java.util.List;

/**
 * Created by IDIC on 2017/2/22.
 */
public class Device implements Parcelable {

    private String UDN;
    private String name;
    private Type type;
    private Manufacturer manufacturer;
    private Model model;
    private String UPC;
    private List<Icon> icons = new LinkedList<>();

    public Device() {
    }

    private Device(Parcel in) {
        UDN = in.readString();
        name = in.readString();
        type = Device.Type.valueOf(in.readString());
        manufacturer = in.readParcelable(Manufacturer.class.getClassLoader());
        model = in.readParcelable(Model.class.getClassLoader());
        UPC = in.readString();
        icons = in.createTypedArrayList(Icon.CREATOR);
    }

    @Override
    public void writeToParcel(Parcel dest, int flags) {
        dest.writeString(UDN);
        dest.writeString(name);
        dest.writeString(type.name());
        dest.writeParcelable(manufacturer, flags);
        dest.writeParcelable(model, flags);
        dest.writeString(UPC);
        dest.writeTypedList(icons);
    }

    @Override
    public int describeContents() {
        return 0;
    }

    public static final Creator<Device> CREATOR = new Creator<Device>() {
        @Override
        public Device createFromParcel(Parcel in) {
            return new Device(in);
        }

        @Override
        public Device[] newArray(int size) {
            return new Device[size];
        }
    };

    public String getUDN() {
        return UDN;
    }

    public Device setUDN(String UDN) {
        this.UDN = UDN;
        return this;
    }

    public Type getType() {
        return type;
    }

    public Device setType(Type type) {
        this.type = type;
        return this;
    }

    public String getName() {
        return name;
    }

    public Device setName(String name) {
        this.name = name;
        return this;
    }

    public Manufacturer getManufacturer() {
        return manufacturer;
    }

    public Device setManufacturer(Manufacturer manufacturer) {
        this.manufacturer = manufacturer;
        return this;
    }

    public Model getModel() {
        return model;
    }

    public Device setModel(Model model) {
        this.model = model;
        return this;
    }

    public String getUPC() {
        return UPC;
    }

    public Device setUPC(String UPC) {
        this.UPC = UPC;
        return this;
    }

    public List<Icon> getIcons() {
        return icons;
    }

    public Device setIcons(List<Icon> icons) {
        this.icons = icons;
        return this;
    }

    public Device addIcon(Icon icon) {
        if (null != this.icons) this.icons.add(icon);
        return this;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Device device = (Device) o;

        if (UDN != null ? !UDN.equals(device.UDN) : device.UDN != null) return false;
        if (name != null ? !name.equals(device.name) : device.name != null) return false;
        if (type != device.type) return false;
        if (manufacturer != null ? !manufacturer.equals(device.manufacturer) : device.manufacturer != null)
            return false;
        if (model != null ? !model.equals(device.model) : device.model != null) return false;
        if (UPC != null ? !UPC.equals(device.UPC) : device.UPC != null) return false;
        return icons != null ? icons.equals(device.icons) : device.icons == null;
    }

    @Override
    public int hashCode() {
        int result = UDN != null ? UDN.hashCode() : 0;
        result = 31 * result + (name != null ? name.hashCode() : 0);
        result = 31 * result + (type != null ? type.hashCode() : 0);
        result = 31 * result + (manufacturer != null ? manufacturer.hashCode() : 0);
        result = 31 * result + (model != null ? model.hashCode() : 0);
        result = 31 * result + (UPC != null ? UPC.hashCode() : 0);
        result = 31 * result + (icons != null ? icons.hashCode() : 0);
        return result;
    }

    public enum Type {
        Sensor, Actuator, Mixed, Other
    }
}
