package org.prlab.idic.privacypolicy.storage;

import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri;
import android.support.annotation.NonNull;

import static org.prlab.idic.privacypolicy.storage.PrivacyPolicyContract.PolicyEntry.TABLE_NAME;

/**
 * Created by IDIC on 2017/2/18.
 */
public class PrivacyPolicyProvider extends ContentProvider {

    public static final int CODE_PRIVACY_POLICY = 100;

    /*
     * The URI Matcher used by this content provider. The leading "s" in this variable name
     * signifies that this UriMatcher is a static member variable of WeatherProvider and is a
     * common convention in Android programming.
     */
    private static final UriMatcher sUriMatcher = buildUriMatcher();
    private PrivacyPolicyDbHelper mOpenHelper;

    @Override
    public boolean onCreate() {
        mOpenHelper = new PrivacyPolicyDbHelper(getContext());
        return true;
    }

    @Override
    public Cursor query(Uri uri, String[] projection, String selection, String[] selectionArgs, String sortOrder) {
        final SQLiteDatabase db = mOpenHelper.getReadableDatabase();
        Cursor cursor;
        switch (sUriMatcher.match(uri)) {
            case CODE_PRIVACY_POLICY: {
                cursor = db.query(
                        TABLE_NAME,
                        projection,
                        selection,
                        selectionArgs,
                        null,
                        null,
                        sortOrder);
                break;
            }
            default:
                throw new UnsupportedOperationException("Unknown uri: " + uri);
        }
        cursor.setNotificationUri(getContext().getContentResolver(), uri);
        return cursor;
    }

    @Override
    public String getType(Uri uri) {
        return null;
    }

    @Override
    public Uri insert(@NonNull Uri uri, @NonNull ContentValues values) {
        final SQLiteDatabase db = mOpenHelper.getWritableDatabase();
        switch (sUriMatcher.match(uri)) {
            case CODE_PRIVACY_POLICY:
                db.beginTransaction();
                try {
                    long _id = db.insertWithOnConflict(TABLE_NAME, null, values, SQLiteDatabase.CONFLICT_IGNORE);
                    if (_id == -1) break;
                    db.setTransactionSuccessful();
                    Uri curr = ContentUris.withAppendedId(PrivacyPolicyContract.PolicyEntry.CONTENT_URI, _id);
                    //將新插入的資料uri進行通知，以便獲取最新的資料
                    getContext().getContentResolver().notifyChange(curr, null);
                    return curr;
                } finally {
                    db.endTransaction();
                }
        }
        return null;
    }

    @Override
    public int delete(Uri uri, String selection, String[] selectionArgs) {

        /* Users of the delete method will expect the number of rows deleted to be returned. */
        int numRowsDeleted;

        /*
         * If we pass null as the selection to SQLiteDatabase#delete, our entire table will be
         * deleted. However, if we do pass null and delete all of the rows in the table, we won't
         * know how many rows were deleted. According to the documentation for SQLiteDatabase,
         * passing "1" for the selection will delete all rows and return the number of rows
         * deleted, which is what the caller of this method expects.
         */
        if (null == selection) selection = "1";

        final SQLiteDatabase db = mOpenHelper.getWritableDatabase();
        db.beginTransaction();
        switch (sUriMatcher.match(uri)) {
            case CODE_PRIVACY_POLICY:
                numRowsDeleted = db.delete(
                        TABLE_NAME,
                        selection,
                        selectionArgs);
                db.setTransactionSuccessful();
                break;
            default:
                throw new UnsupportedOperationException("Unknown uri: " + uri);
        }

        db.endTransaction();
        /* If we actually deleted any rows, notify that a change has occurred to this URI */
        if (numRowsDeleted != 0) {
            getContext().getContentResolver().notifyChange(uri, null);
        }

        return numRowsDeleted;
    }

    @Override
    public int update(Uri uri, ContentValues values, String selection, String[] selectionArgs) {
        final SQLiteDatabase db = mOpenHelper.getWritableDatabase();
        int count;
        switch (sUriMatcher.match(uri)) {
            case CODE_PRIVACY_POLICY:
                db.beginTransaction();
                count = db.update(TABLE_NAME,
                        values,
                        selection,
                        selectionArgs);
                db.setTransactionSuccessful();
                db.endTransaction();
                break;
            default:
                throw new UnsupportedOperationException("Unknown uri: " + uri);
        }
        getContext().getContentResolver().notifyChange(uri, null);
        return count;
    }

    private static UriMatcher buildUriMatcher() {
        final UriMatcher matcher = new UriMatcher(UriMatcher.NO_MATCH);
        final String authority = PrivacyPolicyContract.CONTENT_AUTHORITY;
        matcher.addURI(authority, PrivacyPolicyContract.PATH_PRIVACY_POLICY, CODE_PRIVACY_POLICY);
        return matcher;
    }

}
