package org.prlab.idic.privacypolicy.tool;

import org.prlab.idic.privacypolicy.pojo.report.Layer;
import org.prlab.idic.privacypolicy.pojo.report.PrivacyPolicy;
import org.prlab.idic.privacypolicy.pojo.report.p3p.Remedy;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * Created by IDIC on 2017/1/20.
 */

public class BaseUtils {

    private BaseUtils() {
    }

    public static String bytesToHex(byte[] in) {
        if (in == null || in.length == 0) return "null";
        final StringBuilder builder = new StringBuilder();
        if (in.length > 0) builder.append("0x");
        for (byte b : in) builder.append(String.format("%02x", b));
        return builder.toString();
    }

    public static boolean isArrayBeginWith(final byte[] array, final byte[] prefix) {
        if (array.length < prefix.length) {
            return false;
        }
        for (int i = 0; i < prefix.length; i++) {
            if (array[i] != prefix[i]) {
                return false;
            }
        }
        return true;
    }

    public static int getIntFromByte(final byte bite) {
        return bite & 0xFF;
    }

    public static String convertRemedies2String(List<Remedy> enums) {
        if (enums.size() <= 0) return "";
        StringBuilder builder = new StringBuilder();
        for (Enum e : enums) {
            builder.append(e.name()).append(", ");
        }
        builder.delete(builder.length() - 2, builder.length());
        return builder.toString();
    }

    private static final String SPLIT_STRING = ", ";

    public static List<String> arrayString2stringArray(String arrayString) {
        return BaseUtils.arrayString2objectArray(arrayString, new BaseUtils.ObjectConverter<String>() {
                    @Override
                    public String convert(String string) {
                        return string;
                    }
                }
        );
    }

    public static List<Integer> arrayString2intArray(String arrayString) {
        return BaseUtils.arrayString2objectArray(arrayString, new BaseUtils.ObjectConverter<Integer>() {
                    @Override
                    public Integer convert(String string) {
                        return Integer.parseInt(string);
                    }
                }
        );
    }

    public static <T> List<T> arrayString2objectArray(String arrayString, ObjectConverter<T> converter) {
        List<T> list = new ArrayList<>();
        if (arrayString != null) {
            String[] splits = arrayString.split(SPLIT_STRING);
            for (String split : splits) {
                list.add(converter.convert(split));
            }
        }
        return list;
    }

    public interface ObjectConverter<T> {
        T convert(String string);
    }

    public static String intList2ArrayString(List<Integer> integers) {
        return BaseUtils.objectArray2ArrayString(integers);
    }

    public static String stringList2ArrayString(List<String> strings) {
        return BaseUtils.objectArray2ArrayString(strings);
    }

    public static <T> String objectArray2ArrayString(List<T> objects) {
        StringBuilder builder = new StringBuilder();
        return objects != null ?
                builder.append(String.valueOf(objects)).deleteCharAt(0).deleteCharAt(builder.length() - 1).toString() : "";
    }

    public static String shortMAC(String mac) {
        return mac.replaceAll(":", "");
    }

    public static String fixTypeName(String name) {
        if (name.length() <= 1) return "";
        name = name.replaceAll("_", " ");
        StringBuilder builder = new StringBuilder(name);
        builder.delete(1, builder.length());
        builder.append(name.toLowerCase());
        builder.delete(1, 2);
        return builder.toString();
    }

    public static List<String> layer2UUID(List<Layer> layers) {
        List<String> UUIDs = new LinkedList<>();
        if (null == layers || layers.isEmpty()) return UUIDs;
        for (Layer layer : layers)
            UUIDs.add(layer.getUUID());
        return UUIDs;
    }

    public static byte[] int2Bytes(int i) {
        return ByteBuffer.allocate(4).putInt(i).array();
    }

    public static int bytesToInt(byte[] bytes) {
        return ByteBuffer.wrap(bytes).getInt();
    }

}
