package org.prlab.idic.privacypolicy.tool.beacon;

import org.prlab.idic.privacypolicy.tool.BaseUtils;

/**
 * Created by IDIC on 2017/2/14.
 */
public final class BeaconUtils {

    public static final byte[] MANUFACTURER_DATA_IBEACON_PREFIX = {0x4C, 0x00, 0x02, 0x15};

    private BeaconUtils() {
        // TO AVOID INSTANTIATION
    }

    /**
     * Ascertains whether a Manufacturer Data byte array belongs to a known Beacon type;
     *
     * @param manufacturerData a Bluetooth LE device's raw manufacturerData.
     * @return the {@link BeaconType}
     */
    public static BeaconType getBeaconType(final byte[] manufacturerData) {
        if (manufacturerData == null || manufacturerData.length == 0) {
            return BeaconType.NOT_A_BEACON;
        }

        if (isBeacon(manufacturerData)) {
            return BeaconType.NORMAL_BEACON;
        } else {
            return BeaconType.NOT_A_BEACON;
        }
    }

    private static boolean isBeacon(byte[] manufacturerData) {
        // An iBeacon record must be at least 25 chars long
        return manufacturerData.length >= 23;
    }

    public static String calculateUuidString(final byte[] uuid) {
        final StringBuilder sb = new StringBuilder();

        for (int i = 0; i < uuid.length; i++) {
            if (i == 4 | i == 6 | i == 8 | i == 10) {
                sb.append('-');
            }

            final int intFromByte = BaseUtils.getIntFromByte(uuid[i]);
            if (intFromByte <= 0xF) {
                sb.append('0');
            }
            sb.append(Integer.toHexString(intFromByte));
        }
        return sb.toString();
    }
}